; Boot up the calculator
; We make sure all the hardware is in order and that's it.
; Put your code at the bottom of this file, there's a little marker indicating where.

boot:
    di
    #ifdef CPU15
    ; Set memory mapping
    ; Bank 0: Flash Page 00
    ; Bank 1: Flash Page *
    ; Bank 2: RAM Page 01
    ; Bank 3: RAM Page 00 ; In this order for consistency with TI-83+ and TI-73 mapping
    ld a, 0x81
    out (7), a
    #else
    ; Set memory mapping
    ; Bank 0: Flash Page 00
    ; Bank 1: Flash Page *
    ; Bank 2: RAM Page 01
    ; Bank 3: RAM Page 00
    ld a, 0x41
    out (7), a
    #endif
    
    ld sp, 0xFFFF
    call suspendDevice ; Turns off the screen, enters low power mode, and waits for [ON]
    
reboot:
    ; Runs after the user presses ON
    ; We re-do some things from above to make sure the system is clean when someone does
    ; jp reboot
    di
    ld sp, 0xFFFF
    
    ; Re-map memory
    ld a, 6
    out (4), a
    #ifdef CPU15
    ld a, 0x81
    out (7), a
    #else
    ld a, 0x41
    out (7), a
    #endif
    
    ; Disables Flash protection, RAM protection
    #ifdef CPU15 ; TI-83+ SE, TI-84+, TI-84+ SE
        call unlockFlash
            ; Remove RAM Execution Protection
            xor a
            out (0x25), a ; RAM Lower Limit
            dec a
            out (0x26), a ; RAM Upper Limit

            ; Remove Flash Execution Protection
            out (0x23), a ; Flash Upper Limit
            out (0x22), a ; Flash Lower Limit
        call lockFlash

        ; Set CPU speed to 15 MHz
        ld a, 1
        out (0x20), a
    
    #else ; TI-73, TI-83+
        #ifndef TI73 ; RAM does not have protection on the TI-73
        
        ; Remove RAM/Flash protection
        call unlockFlash
            xor a
            out (5), a
            out (0x16), a

            ld a, 0b00000001
            out (5), a
            xor a
            out (0x16), a

            ld a, 0b00000010
            out (5), a
            xor a
            out (0x16), a

            ld a, 0b00000111
            out (5), a
            xor a
            out (0x16), a
        call lockFlash
        #endif
    #endif
    
    ; Set intterupt mode
    ld a, 0b00001011 ; Same as TIOS, different than KnightOS
    out (3), a
    
    ; Initialize LCD
    ld a, 5
    call lcdDelay
    out (0x10), a ; X-Increment Mode

    ld a, 1
    call lcdDelay
    out (0x10), a ; 8-bit mode

    ld a, 3
    call lcdDelay
    out (0x10), a ; Enable screen

    ld a, 0x17
    call lcdDelay
    out (0x10), a ; Op-amp control (OPA1) set to max (with DB1 set for some reason)

    ld a, 0xB ; B
    call lcdDelay
    out (0x10), a ; Op-amp control (OPA2) set to max
    
    ; The best contrast varies from model to model
    #ifdef USB
        ld a, 0xEF
    #else
        #ifdef TI73
            ld a, 0xFB
        #else
            ld a, 0xF4
        #endif
    #endif
    call lcdDelay
    out (0x10), a ; Contrast
    
    ;;;;;
    ; At this point, we've booted up successfully. The screen is on, memory is mapped.
    ; This is where you should add your own code.
    ;;;;;
    
	ld de,0
	ld (smallY),de
	ld (prevSmallY),de
	ld iy,graphBuf
	ld hl,welcomeString
	call printString
loop:
	ld hl,inputCommand
	call printString
	call beginCmdInput
	call parseInput
	
	call waitNoKey
	jr loop

inputCommand:
	.db ">> ",0
	
welcomeString:
	.db "cmdOS v0.0\n",0
