; Command input functions

; Inputs :
;	(smallX) = X coordinate of the beginning of the input
;	(smallY) = Y coordinate
;	iy = buffer
; Outputs :
;	writes what the user types in the buffer
;	command copied in (inputBuf)
beginCmdInput:
	saveRegs

	ld hl,0
	ld (inputOffset),hl
	ld hl,inputBufCharSec
	ld a,0
	ld (inputBufXposSec),a
	ld (inputBufYposSec),a
	ld bc,128
	call memset
_beginCmdInputLoop:
	call fastCopy
	call waitNoKey
	call waitForKey
	
	ld hl,keys
	ld e,a
	ld d,0
	add hl,de
	ld a,(hl)
		
	ld hl,(inputOffset)
	ld de,inputBufCharSec
	add hl,de
	
	; handles backspace
	cp $FD
	jr nz,$+7
	call handleBackSpace
	jr _beginCmdInputLoop
	
	; handles inactive keys
	cp $FF
	jr z,_beginCmdInputLoop
	
	; stores the inputted character
	ld (hl),a

	push af
		ld hl,(inputOffset)
		push hl
			ld de,inputBufXposSec
			add hl,de
			ld a,(smallX)
			ld (hl),a
		pop hl
		ld de,inputBufYposSec
		add hl,de
		ld a,(smallY)
		ld (hl),a
	pop af

	call putC

	; terminates the input
	cp '\n'
	jr z,_beginCmdInputEnd
	
	ld hl,(inputOffset)
	inc hl
	ld (inputOffset),hl
	
	jr _beginCmdInputLoop
	
_beginCmdInputEnd:
	ld (hl),0
	
	restoreRegs
	ret

keys:
	; $FF = inactive key
	; $FE = toggle alpha mode
	; $FD = backspace
	; $FC = erase line
	; '\n' = finish input
	.db $FF
	.db $FF,$FF,$FF,$FF
	.db $FF,$FF,$FF,$FF
	.db '\n','"','W','R','M','H',$FC
	.db $FF
	.db '_','#','V','Q','L','G',$FF
	.db $FF
	.db '.','Z','U','P','K','F','C',$FF
	.db ' ','Y','T','O','J','E','B',$FF
	.db $FF,'X','S','N','I','D','A',$FE
	.db $FF,$FF,$FF,$FF,$FF
	.db $FF,$FF,$FD
	
; Inputs :
;	hl = pointer on character to erase + 1
;	iy = buffer
; Outputs :
;	character erased on buffer
;	hl = pointer on character erased
;	smallX and smallY updated with previous values
handleBackSpace:
	saveRegs
	dec hl
	push hl
		ld hl,(inputOffset)
		ld a,l
		or h
	pop hl
	jr nz,_handleBackSpace
	
	restoreRegs
	ret
	
_handleBackSpace:
	ld a,(hl)
	call getCharMap
	ld b,(hl)
	ld hl,(inputOffset)
	dec hl
	push hl
		ld de,inputBufXposSec
		add hl,de
		ld a,(hl)
		ld (smallX),a
		ld c,a
	pop hl
	ld de,inputBufYposSec
	add hl,de
	ld a,(hl)
	ld (smallY),a
	ld d,b
	ld e,c
	ld l,a
	ld h,5
	ld a,2
	call drawRect
	
	ld hl,(inputOffset)
	dec hl
	ld (inputOffset),hl
	ld de,inputBufCharSec
	add hl,de
	ld (hl),0
	restoreRegs
	ret
	
; Inputs :
;	(inputBufCharSec) = start of command to parse
; Outputs:
;	string parsed and action made accordingly
parseInput:
	saveRegs
	
	ld hl,inputBufCharSec
	ld b,' '
	push hl
		call seekChar
	pop hl
	or a
	jr z,_parseInputNoArgCommand
	
	; parse commands with arguments first
	push hl
		push bc
			call seekChar
			inc hl
			ld de,inputBufArgsSec
			push hl \ call getStrLength \ pop hl
			ld b,0
			ldir
		pop bc
	pop hl
	
	push hl
		ld de,echoCmd
		call compareStringsUntilB
	pop hl
	jr nz,$+7
	call execEcho
	jr _parseInputEnd
	
	push hl
		ld de,fileInfoCmd
		call compareStringsUntilB
	pop hl
	jr nz,$+7
	call execFileInfo
	jr _parseInputEnd
	
	jr _parseInputEnd
	
	; if no one matches, parse command with arguments
_parseInputNoArgCommand:
	push hl
		ld de,shutdownCmd
		call compareZeroTerminatedStrings
	pop hl
	jr nz,$+7
	call execShutdown
	jr _parseInputEnd
	
	push hl
		ld de,clrIOCmd
		call compareZeroTerminatedStrings
	pop hl
	jr nz,$+7
	call execClrIO
	jr _parseInputEnd
	
	push hl
		ld de,pingCmd
		call compareZeroTerminatedStrings
	pop hl
	jr nz,$+7
	call execPing
	jr _parseInputEnd
	
	push hl
		ld de,versionCmd
		call compareZeroTerminatedStrings
	pop hl
	jr nz,$+7
	call execVersion
	jr _parseInputEnd
	
	call printString

_parseInputEnd:
	restoreRegs
	ret

; Commands strings
; no args commands
shutdownCmd:
	.db "SHUTDOWN\n",0
clrIOCmd:
	.db "CLRIO\n",0
pingCmd:
	.db "PING\n",0
versionCmd:
	.db "VERSION\n",0
; commands taking arguments
echoCmd:
	.db "ECHO ",0
fileInfoCmd:
	.db "FILEINFO ",0