; Graphical routines

; Inputs :
;   iy = buffer
; Outputs :
;   Buffer copied to screen
fastCopy:
	saveRegs
	
	push iy \ pop hl
	ld c,$80
	ld b,$20
_fastCopyLoop:
	ld a,b
	call lcdDelay
	out ($10),a
	ld a,c
	call lcdDelay
	out ($10),a
	ld a,(hl)
	inc hl
	call lcdDelay
	out ($11),a
	inc b
	ld a,b
	cp $2C
	jr c,_fastCopyLoop
	inc c
	ld a,c
	cp $C0
	jr c,_fastCopyLoop-2
	
	restoreRegs
	ret

; Inputs :
;   iy = buffer
; Outputs :
;   768 bytes of buffer filled with 0
clearBuf:
	saveRegs
	
	push iy \ pop hl
	ld (hl),0
	push hl \ pop de
	inc de
	ld bc,767
	ldir
	
	restoreRegs
	ret
	
; Inputs :
;	e = X coordinate
;	l = Y coordinate
;	iy = buffer
; Outputs :
;	a = pixel mask
;	hl = pointer on byte
getPixelMask:
	push bc
	push de
	
	ld d,0
	ld h,0
	add hl,hl
	add hl,hl
	push hl \ pop bc
	add hl,hl
	add hl,bc
	push iy \ pop bc
	add hl,bc
	ld a,e
	srl e
	srl e
	srl e
	add hl,de
	
	and 7
	ld b,a
	ld a,$80
	
	rrca
	djnz $-1
	
	pop de
	pop bc
	ret
	
; Inputs :
;   e = X coordinate
;   l = Y coordinate
;   iy = buffer
;	a = drawing mode : 0 -> black, 1 -> xor, 2 -> white
; Outputs:
;   pixel drawn to buffer
setPixel:
	saveRegs
	ld b,a
	call getPixelMask
	
	ld d,a
	ld a,b
	cp 1
	ld a,d
	jr c,_setPixelBlack
	jr z,_setPixelXor
	cpl
	and (hl)
	ld (hl),a
	jr _setPixelEnd
_setPixelBlack:
	or (hl)
	ld (hl),a
	jr _setPixelEnd
_setPixelXor:
	xor (hl)
	ld (hl),a
_setPixelEnd:

	restoreRegs
	ret

; Ripped from KnightOS
; D = xpos
; E = ypos
; B = height
; IX = image address
; IY = buffer address
PutSpriteOR:
	saveRegs
        push hl \ pop ix
        call _ClipSprOR
    restoreRegs
    ret
    
_ClipSprOR:
; Start by doing vertical clipping
    LD     A, %11111111         ; Reset clipping mask
    LD     (clip_mask), A
    LD     A, E                 ; If ypos is negative
    OR     A                    ; try clipping the top
    JP     M, ClipTop3           ;
 
    SUB    64                   ; If ypos is >= 64
    RET    NC                   ; sprite is off-screen

    NEG                         ; If (64 - ypos) > height
    CP     B                    ; don't need to clip
    JR     NC, VertClipDone3     ; 

    LD     B, A                 ; Do bottom clipping by
    JR     VertClipDone3         ; setting height to (64 - ypos)

ClipTop3:
    LD     A, B                 ; If ypos <= -height
    NEG                         ; sprite is off-screen
    SUB    E                    ;
    RET    NC                   ;

    PUSH   AF
    ADD    A, B                 ; Get the number of clipped rows
    LD     E, 0                 ; Set ypos to 0 (top of screen)
    LD     B, E                 ; Advance image data pointer
    LD     C, A                 ;
    ADD    IX, BC               ;
    POP    AF
    NEG                         ; Get the number of visible rows
    LD     B, A                 ; and set as height

VertClipDone3:
; Now we're doing horizontal clipping
    LD     C, 0                 ; Reset correction factor
    LD     A, D

    CP     -7                   ; If 0 > xpos >= -7
    JR     NC, ClipLeft3         ; clip the left side

    CP     96                   ; If xpos >= 96
    RET    NC                   ; sprite is off-screen

    CP     89                   ; If 0 <= xpos < 89
    JR     C, HorizClipDone3     ; don't need to clip

ClipRight3:
    AND    7                    ; Determine the clipping mask
    LD     C, A
    LD     A, %11111111
FindRightMask3:
    ADD    A, A
    DEC    C
    JR     NZ, FindRightMask3
    LD     (clip_mask), A
    LD     A, D
    JR     HorizClipDone3

ClipLeft3:
    AND    7                    ; Determine the clipping mask
    LD     C, A
    LD     A, %11111111
FindLeftMask3:
    ADD    A, A
    DEC    C
    JR     NZ, FindLeftMask3
    CPL
    LD     (clip_mask), A
    LD     A, D
    ADD    A, 96                ; Set xpos so sprite will "spill over"
    LD     C, 12                ; Set correction

HorizClipDone3:
; A = xpos
; E = ypos
; B = height
; IX = image address

; Now we can finally display the sprite.
    LD     H, 0
    LD     D, H
    LD     L, E
    ADD    HL, HL
    ADD    HL, DE
    ADD    HL, HL
    ADD    HL, HL

    LD     E, A
    SRL    E
    SRL    E
    SRL    E
    ADD    HL, DE

    push iy \ pop de ; LD     DE, PlotSScreen
    ADD    HL, DE

    LD     D, 0                 ; Correct graph buffer address
    LD     E, C                 ; if clipping the left side
    SBC    HL, DE               ;

    AND    7
    JR     Z, _Aligned3

    LD     C, A
    LD     DE, 11

_RowLoop3:
    PUSH   BC
    LD     B, C
    LD     A, (clip_mask)       ; Mask out the part of the sprite
    AND    (IX)                 ; to be horizontally clipped
    LD     C, 0

_ShiftLoop3:
    SRL    A
    RR     C
    DJNZ   _ShiftLoop3

    OR    (HL)
    LD     (HL), A

    INC    HL
    LD     A, C
    OR    (HL)
    LD     (HL), A

    ADD    HL, DE
    INC    IX
    POP    BC
    DJNZ   _RowLoop3
    RET

_Aligned3:
    LD     DE, 12

_PutLoop3:
    LD     A, (IX)
    OR    (HL)
    LD     (HL), A
    INC    IX
    ADD    HL, DE
    DJNZ   _PutLoop3
    RET
	
; Inputs :
;	iy = buffer
; Outputs :
;   buffer shifted up 6 pixels
shiftBufUp6px:
	saveRegs
	
	push iy \ pop hl
	push hl
		ld de,6*12
		add hl,de
	pop de
	ld bc,58*12
	ldir
	
	push iy \ pop hl
	ld de,58*12
	add hl,de
	push hl \ pop de
	inc de
	ld (hl),0
	ld bc,6*12
	ldir
	
	restoreRegs
	ret

; Inputs :
;	e = X coordinate of top left
;	l = Y coordinate of top left
;	d = width
;	h = height
;	iy = buffer
;	a = drawing mode : 0 -> black, 1 -> xor, 2 -> white
; Outputs :
;	White rectangle drawn on buffer
drawRect:
	saveRegs
	
	ld b,h
	ld c,e
_drawRectYloop:
	push bc
		ld b,d
		ld e,c
_drawRectXloop:
		call setPixel
		inc e
		djnz _drawRectXloop
	pop bc
	inc l
	djnz _drawRectYloop
	restoreRegs
	ret
