; Text-related funcs

; Inputs :
;	a = character
; Outputs :
;	a unchanged
;	hl = pointer on width byte of character in fontmap
getCharMap:
	sub $20 ; excludes command characters of ANSI set	
	ld l,a
	ld h,0
	push hl \ pop bc
	add hl,hl
	add hl,bc
	add hl,hl
	ld bc,Font
	add hl,bc
	ret
	
; Inputs :
;   (smallX) = X coordinate
;   (smallY) = Y coordinate
;	a = character
;	iy = buffer
; Outputs :
;   character printed to buffer with small font
;	smallX and smallY updated
;	buffer scrolled if needed
putC:
	saveRegs
	
	ld b,a
	; Handles newline
	cp '\n'
	jr z,_putCNewLineChar
	ld a,(smallX)
	cp 90
	jr c,_putCNoNewLine
_putCNewLineChar:
	ld hl,(smallY)
	ld h,0
	ld a,6
	add a,l
	ld l,a
	ld (smallY),hl
	
	ld a,b
	cp '\n'
	jr nz,_putCNoNewLine
	
	restoreRegs
	ret
_putCNoNewLine:
	; check for boundaries
	ld a,(smallY)
	cp 60
	jr c,_putCNoWrap
	call shiftBufUp6px
	sub 6
	ld (smallY),a
_putCNoWrap:
	ld a,b
	call getCharMap
	inc hl
	ld b,5
	ld de,(smallY)
	push hl \ pop ix
	push ix
		call PutSpriteOR
	pop hl

	dec hl
	ld a,(smallX)
	add a,(hl)
	ld (smallX),a

	restoreRegs
	ret

; Inputs :
;	(smallX) = X coordinate
;	(smallY) = Y coordinate
;	hl = pointer to string
;	iy = buffer
; Outputs :
;	String printed to buffer with small font
printString:
	saveRegs

_printStringLoop:
	ld a,(hl)
	or a
	jr z,_printStringEnd
	call putC
	inc hl
	jr _printStringLoop
	
_printStringEnd:
	restoreRegs
	ret

; Inputs :
;	iy = buffer
; Outputs :
;	buffer erased
;	smallX and smallY reset
clearTextBuf:
	saveRegs
	
	call clearBuf
	
	ld hl,0
	ld (smallY),hl
	
	restoreRegs
	ret

; Inputs :
;	hl = pointer on first string
;	de = pointer on second string
; Outputs :
;	hl = pointer on first unmatching character of first string or end of string if strings match
;	de = same for second string
;	z = set if strings match, unset otherwise
compareZeroTerminatedStrings:
	ld a,(de)
	cp (hl)
	ret nz
	or a
	ret z
	inc hl
	inc de
	jr compareZeroTerminatedStrings
	
; Inputs :
;	hl = pointer on first string
;	de = pointer on second string
;	b = character the breaks the compare if encountered IN THE STRING IN HL ONLY
; Outputs :
;	hl = pointer on the last character of the first string tested 
;	de = same for second string
;	z = set if all the tested bytes match, unset otherwise
;	a = last character of string in HL tested
;	c = same for string in DE
; Note :
;	The compare stops as soon as the character in B or a zero is encountered IN HL. It will also break if (hl) and (de) don't match.
compareStringsUntilB:
	ld a,(de)
	ld c,a
	ld a,(hl)
	cp c
	ret nz
	cp b
	ret z
	or a
	ret z
	inc hl
	inc de
	jr compareStringsUntilB

; Inputs :
;	hl = pointer on string
; Outputs :
;	c = size of the string, zero included
;	hl = pointer on end of string
getStrLength:
	ld c,0
	inc c
	ld a,(hl)
	or a
	ret z
	inc hl
	jr getStrLength+2

; Inputs :
;	hl = pointer on string
;	b = breaking character
; Outputs :
;	hl = pointer on breaking character or end of string
;	a = character that broke the search (can be 0)
seekChar:
	ld a,(hl)
	cp b
	ret z
	or a
	ret z
	inc hl
	jr seekChar
	
; Inputs :
;	smallX = X coordinate
;	smallY = Y coordinate
;	a = hex to print
; Outputs :
;	a printed in hex to the screen
;	smallX and smallY updated
printHex:
	saveRegs
	
	ld b,a
	srl a
	srl a
	srl a
	srl a
	ld hl,hexChars
	push hl
		ld e,a
		ld d,0
		add hl,de
		ld a,(hl)
		call putC
	pop hl
	ld a,b
	and 15
	ld e,a
	add hl,de
	ld a,(hl)
	call putC
	
	restoreRegs
	ret

hexChars:
	.db "0123456789ABCDEF"